@@ -25,7 +25,7 @@ slave.hostinfo <- function(comm=1, short=TRUE){
     else { 
         master <-mpi.get.processor.name() 
         slavehost <- unlist(mpi.remote.exec(mpi.get.processor.name(),comm=comm))
-        slavecomm <- as.integer(mpi.remote.exec(.comm,comm=comm))
+        slavecomm <- 1 #as.integer(mpi.remote.exec(.comm,comm=comm))
         ranks <- 1:(size-1)
         commm <- paste(comm, ")",sep="")
         if (size > 10){
@@ -117,14 +117,16 @@ mpi.spawn.Rslaves <-
     comm=1,
     hosts=NULL,
     needlog=TRUE,
-    mapdrive=TRUE) {
+    mapdrive=TRUE,
+	quiet=FALSE,
+	nonblock=TRUE,
+	sleep=0.1) {
     if (!is.loaded("mpi_comm_spawn"))
         stop("You cannot use MPI_Comm_spawn API")   
     if (mpi.comm.size(comm) > 0){
          err <-paste("It seems there are some slaves running on comm ", comm)
          stop(err)
     }
-      #  Rslavecmd<-ifelse(.Platform$OS=="windows","Rslalves.cmd","Rslaves.sh")
     if (.Platform$OS=="windows"){
         workdrive <- unlist(strsplit(getwd(),":"))[1]
 		workdir <- unlist(strsplit(getwd(),"/"))
@@ -134,18 +136,18 @@ mpi.spawn.Rslaves <-
         	workdir <- paste(workdir,"\\")
         localhost <- Sys.getenv("COMPUTERNAME")
         networkdrive <-.Call("RegQuery", as.integer(2),paste("NETWORK\\",workdrive,sep=""), 
-                        PACKAGE="npRmpi")
+                        PACKAGE = "npRmpi")
         remotepath <-networkdrive[which(networkdrive=="RemotePath")+1]
         mapdrive <- as.logical(mapdrive && !is.null(remotepath))
         arg <- c(Rscript, R.home(), workdrive, workdir, localhost, mapdrive, remotepath)
-		if (getRversion() >= "2.12.0")
-          realns <- mpi.comm.spawn(slave = system.file("Rslaves32.bat", 
-            package = "npRmpi"), slavearg = arg, nslaves = nslaves, 
-            info = 0, root = root, intercomm = intercomm)
+		if (.Platform$r_arch == "i386") 
+            realns <- mpi.comm.spawn(slave = system.file("Rslaves32.cmd", 
+				package = "npRmpi"), slavearg = arg, nslaves = nslaves, 
+				info = 0, root = root, intercomm = intercomm, quiet = quiet)
 		else 
-	  	  realns <- mpi.comm.spawn(slave = system.file("Rslaves.bat", 
-            package = "npRmpi"), slavearg = arg, nslaves = nslaves, 
-            info = 0, root = root, intercomm = intercomm)
+			realns <- mpi.comm.spawn(slave = system.file("Rslaves64.cmd", 
+			            package = "npRmpi"), slavearg = arg, nslaves = nslaves, 
+				info = 0, root = root, intercomm = intercomm, quiet = quiet)
     }
     else{
         tmp <- paste(Sys.getpid(), "+", comm, sep="")   
@@ -172,13 +174,13 @@ mpi.spawn.Rslaves <-
             mpi.info.create(0)
             mpi.info.set(0,"file",tmpfile)
         }
-        realns<-mpi.comm.spawn(slave=system.file("Rslaves.sh", package="npRmpi"),
-        slavearg=arg,
-        nslaves=nslaves,
-        info=0,
-        root=root,
-        intercomm=intercomm)
-    }
+		if (length(unlist(strsplit(.Platform$pkgType,"mac"))) ==2 && .Platform$r_arch =="x86_64")
+			realns<-mpi.comm.spawn(slave=system.file("MacR64slaves.sh", package="npRmpi"),
+			slavearg=arg, nslaves=nslaves, info=0, root=root, intercomm=intercomm, quiet = quiet)
+		else 
+			realns<-mpi.comm.spawn(slave=system.file("Rslaves.sh", package="npRmpi"),
+			slavearg=arg, nslaves=nslaves, info=0, root=root, intercomm=intercomm, quiet = quiet)
+	}
     if (!is.null(hosts)){
         unlink(tmpfile)
         mpi.info.free(0)
@@ -188,23 +190,25 @@ mpi.spawn.Rslaves <-
     if (mpi.intercomm.merge(intercomm,0,comm)) {
         mpi.comm.set.errhandler(comm)
         mpi.comm.disconnect(intercomm)
-        slave.hostinfo(comm)
+		mpi.bcast(nonblock,type=1, rank=0, comm=comm)
+		mpi.bcast(sleep,type=2, rank=0, comm=comm)
+        if (!quiet) slave.hostinfo(comm)
     }
     else
         stop("Fail to merge the comm for master and slaves.")
 }   
 
-mpi.remote.exec <- function(cmd, ...,  comm=1, ret=TRUE){
+mpi.remote.exec <- function(cmd, ...,  simplify=TRUE, comm=1, ret=TRUE){
     if (mpi.comm.size(comm) < 2)
     stop("It seems no slaves running.")
-    tag <- floor(runif(1,1,1000))
+    tag <- floor(runif(1,20000,30000))
     scmd <- substitute(cmd)
     arg <-list(...)
-    if (length(arg) > 0) 
-        deparse(arg)
-    tag.ret <- c(tag, ret)
-    mpi.bcast.cmd(.mpi.slave.exec(), comm = comm)
-    mpi.bcast(as.integer(tag.ret), type=1, comm=comm)
+    #if (length(arg) > 0) 
+    #    deparse(arg)
+    #tag.ret <- c(tag, ret, simplify)
+    mpi.bcast.cmd(.mpi.worker.exec, tag=tag, ret=ret, simplify=simplify, comm = comm)
+    #mpi.bcast(as.integer(tag.ret), type=1, comm=comm)
     mpi.bcast.Robj(list(scmd=scmd, arg=arg), comm=comm)
 
     if (ret){
@@ -214,7 +218,7 @@ mpi.remote.exec <- function(cmd, ...,  comm=1, ret=TRUE){
     len <- allcode[seq(4,2*size,2)]
     eqlen <- all(len==len[1])
     if (all(type==1)){
-        if (eqlen){
+        if (eqlen && simplify){
             out <- mpi.gather(integer(len[1]),1,integer(size*len[1]),0,comm)
             out <- out[(len[1]+1):(size*len[1])]
         dim(out) <- c(len[1], size-1)
@@ -231,7 +235,7 @@ mpi.remote.exec <- function(cmd, ...,  comm=1, ret=TRUE){
         }
     }
     else if (all(type==2)){
-        if (eqlen){
+        if (eqlen && simplify){
                 out <- mpi.gather(double(len[1]),2,double(size*len[1]),0,comm)
                 out <- out[(len[1]+1):(size*len[1])]
                 dim(out) <- c(len[1], size-1)
@@ -248,7 +252,7 @@ mpi.remote.exec <- function(cmd, ...,  comm=1, ret=TRUE){
         }
     }
  else if (all(type==4)){
-        if (eqlen){
+        if (eqlen && simplify){
                 out <- mpi.gather(raw(len[1]),4,raw(size*len[1]),0,comm)
                 out <- out[(len[1]+1):(size*len[1])]
                 dim(out) <- c(len[1], size-1)
@@ -290,22 +294,25 @@ mpi.remote.exec <- function(cmd, ...,  comm=1, ret=TRUE){
             as.integer(-1)
 }
 
-.mpi.slave.exec <- function(){
-    assign(".mpi.err", FALSE,  envir = .GlobalEnv)
-    tag.ret <- mpi.bcast(integer(2), type=1, comm=.comm)
-    tag <- tag.ret[1]
-    ret <- as.logical(tag.ret[2])
+.mpi.worker.exec <- function(tag, ret, simplify){
+    #assign(".mpi.err", FALSE,  envir = .GlobalEnv)
+    assign(".mpi.err", FALSE)
+	.comm <- 1
+    #tag.ret <- mpi.bcast(integer(3), type=1, comm=.comm)
+    #tag <- tag.ret[1]
+    #ret <- as.logical(tag.ret[2])
+    #simplify <- as.logical(tag.ret[3])
     scmd.arg <- mpi.bcast.Robj(comm=.comm)
 
     if (ret){
     size <- mpi.comm.size(.comm)
     myerrcode <- as.integer(0)
     if (length(scmd.arg$arg)>0)
-        out <- try(do.call(as.character(scmd.arg$scmd), scmd.arg$arg),TRUE)
+        out <- try(do.call(as.character(scmd.arg$scmd), scmd.arg$arg, envir=.GlobalEnv),TRUE)
     else
-        out <- try(eval(scmd.arg$scmd), TRUE)
+        out <- try(eval(scmd.arg$scmd, envir=sys.parent()), TRUE)
     
-    if (.mpi.err){
+    if (get(".mpi.err")){
         print(geterrmessage())
         type <- integer(2)
     }
@@ -319,19 +326,19 @@ mpi.remote.exec <- function(cmd, ...,  comm=1, ret=TRUE){
         len <- allcode[seq(4,2*size,2)]
         eqlen <- all(len==len[1])
         if (all(type==1)) {
-            if (eqlen)
+            if (eqlen && simplify)
                 mpi.gather(out, 1, integer(1), 0, .comm)
         else
         mpi.gatherv(out, 1, integer(1), integer(1), 0 ,.comm)
     }
     else if (all(type==2)) {
-            if (eqlen)
+            if (eqlen && simplify)
                 mpi.gather(out, 2, double(1), 0, .comm)
         else
                 mpi.gatherv(out, 2, double(1), integer(1), 0, .comm)
         }
      else if (all(type==4)) {
-            if (eqlen)
+            if (eqlen && simplify)
                 mpi.gather(out, 4, raw(1), 0, .comm)
         else
                 mpi.gatherv(out, 4, raw(1), integer(1), 0, .comm)
@@ -354,7 +361,7 @@ mpi.close.Rslaves <- function(dellog=TRUE, comm=1){
     err <-paste("It seems no slaves running on comm", comm)
     stop(err)
     }
-    mpi.bcast.cmd(break, rank=0, comm=comm)
+    mpi.bcast.cmd(cmd=break, rank=0, comm=comm)
     if (.Platform$OS!="windows"){
         if (dellog && mpi.comm.size(0) < mpi.comm.size(comm)){
         tmp <- paste(Sys.getpid(),"+",comm,sep="")  
@@ -392,8 +399,8 @@ mpi.apply <- function(x, fun, ...,  comm=1){
         stop("fun is not a function")
     length(list(...)) #test for any non existing R objects
     tag <- floor(runif(1,1,1000))    
-    mpi.bcast.cmd(.mpi.slave.apply(),comm=comm)
-    mpi.bcast(as.integer(c(tag,n)),type=1,comm=comm)
+    mpi.bcast.cmd(.mpi.worker.apply, n=n, tag=tag, comm=comm)
+    #mpi.bcast(as.integer(c(tag,n)),type=1,comm=comm)
     mpi.bcast.Robj(list(fun=fun,dot.arg=list(...)),rank=0,comm=comm)
     if (n < nslaves)
         x=c(x,as.list(integer( nslaves-n)))
@@ -408,11 +415,12 @@ mpi.apply <- function(x, fun, ...,  comm=1){
     out
 }
 
-.mpi.slave.apply <- function(){
-    assign(".mpi.err", FALSE,  envir = .GlobalEnv)
-    tag.n <- mpi.bcast(integer(2), type=1, comm=.comm)
-    tag <- tag.n[1]
-    n <- tag.n[2]
+.mpi.worker.apply <- function(n, tag){
+    #assign(".mpi.err", FALSE,  envir = .GlobalEnv)
+	.comm <- 1
+    #tag.n <- mpi.bcast(integer(2), type=1, comm=.comm)
+    #tag <- tag.n[1]
+    #n <- tag.n[2]
     tmpfunarg <- mpi.bcast.Robj(rank=0, comm=.comm)
     .tmpfun <- tmpfunarg$fun
     dotarg <- tmpfunarg$dot.arg
@@ -423,7 +431,7 @@ mpi.apply <- function(x, fun, ...,  comm=1){
     }
 }
 
-mpi.iapply <- function(x, fun, ...,  comm=1, sleep=0.001){
+mpi.iapply <- function(x, fun, ...,  comm=1, sleep=0.01){
     n <- length(x)
     nslaves <- mpi.comm.size(comm)-1
      if (nslaves < n)
@@ -432,8 +440,8 @@ mpi.iapply <- function(x, fun, ...,  comm=1, sleep=0.001){
         stop("fun is not a function")
     length(list(...)) #test for any non existing R objects
     tag <- floor(runif(1,1,1000))    
-    mpi.bcast.cmd(.mpi.slave.apply(),comm=comm)
-    mpi.bcast(as.integer(c(tag,n)),type=1,comm=comm)
+    mpi.bcast.cmd(.mpi.worker.apply, n=n, tag=tag,comm=comm)
+    #mpi.bcast(as.integer(c(tag,n)),type=1,comm=comm)
     mpi.bcast.Robj(list(fun=fun,dot.arg=list(...)),rank=0,comm=comm)
     if (n < nslaves)
         x=c(x,as.list(integer( nslaves-n)))
@@ -446,7 +454,7 @@ mpi.iapply <- function(x, fun, ...,  comm=1, sleep=0.001){
        if (mpi.iprobe(anysource,tag,comm)){ 
        srctag <- mpi.get.sourcetag()
        charlen <- mpi.get.count(type=4)
-           tmp <- .mpi.unserialize(mpi.recv(x = raw(charlen), type = 4, srctag[1], 
+           tmp <- unserialize(mpi.recv(x = raw(charlen), type = 4, srctag[1], 
             srctag[2], comm))
            out[[srctag[1]]]<- tmp
        done=done+1
@@ -461,6 +469,7 @@ mpi.iapply <- function(x, fun, ...,  comm=1, sleep=0.001){
 mpi.parSim <- function(n=100,rand.gen=rnorm, rand.arg=NULL, 
             statistic, nsim=100, run=1, slaveinfo=FALSE, sim.seq=NULL,
             simplify=TRUE, comm=1, ...){
+	sim.seq=NULL
     if (mpi.comm.size(comm) < 2)
         stop("It seems no slaves running.")
     if (!is.function(rand.gen))
@@ -481,12 +490,12 @@ mpi.parSim <- function(n=100,rand.gen=rnorm, rand.arg=NULL,
                 length(sim.seq)!=slave.num*run)
             stop("sim.seq is not in right order")
 
-    mpi.bcast.cmd(.mpi.slave.sim(),comm=comm)  
+    mpi.bcast.cmd(.mpi.worker.sim, n=n, nsim=nsim, run=run, comm=comm)  
     mpi.bcast.Robj(list(rand.gen=rand.gen, rand.arg=rand.arg,
                         stat=statistic, stat.arg=list(...)), comm=comm)
 
-    nnr <- c(n,nsim,run)
-    mpi.bcast(as.integer(nnr),type=1, comm=comm)
+    #nnr <- c(n,nsim,run)
+    #mpi.bcast(as.integer(nnr),type=1, comm=comm)
     result <- as.list(integer(slave.num*run))
 
     if (!is.null(sim.seq)){
@@ -500,30 +509,30 @@ mpi.parSim <- function(n=100,rand.gen=rnorm, rand.arg=NULL,
     i <- 0
     anysrc <- mpi.any.source()
     anytag <- mpi.any.tag()
-    
-    .mpi.parSim <<- integer(slave.num*run)
-
+    mpi.parSim.tmp <- integer(slave.num*run)
     while (i < slave.num*run){
         i <- i+1
         result[[i]] <- mpi.recv.Robj(source=anysrc, tag=8, comm=comm)
         src <- mpi.get.sourcetag()[1]
         mpi.send(as.integer(i), type=1, dest=src, tag=88, comm=comm)
-        .mpi.parSim[i] <<- src
+        mpi.parSim.tmp[i] <- src
     }
     if (slaveinfo){
         slavename <- paste("slave",1:slave.num, sep="")
         cat("Finished slave jobs summary:\n")
         for (i in 1:slave.num){
             if (i < 10)
-                cat(slavename[i], " finished",sum(.mpi.parSim==i), "job(s)\n")
+                cat(slavename[i], " finished",sum(mpi.parSim==i), "job(s)\n")
             else
-                cat(slavename[i], "finished",sum(.mpi.parSim==i), "job(s)\n")
+                cat(slavename[i], "finished",sum(mpi.parSim==i), "job(s)\n")
         }
     }
+	#assign(".mpi.parSim", mpi.parSim.tmp,  envir = .GlobalEnv)
     .simplify(slave.num*run, result, simplify, nsim)
 }
 
-.mpi.slave.sim <- function(){
+.mpi.worker.sim <- function(n, nsim, run){
+	.comm <- 1
     tmpdata <- mpi.bcast.Robj(comm=.comm)
     rand.arg <- tmpdata$rand.arg
     stat.arg <- tmpdata$stat.arg
@@ -531,8 +540,8 @@ mpi.parSim <- function(n=100,rand.gen=rnorm, rand.arg=NULL,
     .tmp.rand.gen <- tmpdata$rand.gen
     .tmp.statistic <- tmpdata$stat
     
-    nnr <- mpi.bcast(integer(3), type=1, comm=.comm)
-    n <- nnr[1];  nsim <- nnr[2];  run <- nnr[3]
+    #nnr <- mpi.bcast(integer(3), type=1, comm=.comm)
+    #n <- nnr[1];  nsim <- nnr[2];  run <- nnr[3]
 
     i <- 0
     slave.num <- mpi.comm.size(.comm)-1
@@ -574,7 +583,7 @@ mpi.parMM <- function(A, B, job.num=mpi.comm.size(comm)-1, comm=1){
         get("%*%"), B, comm=comm))
 }
     
-mpi.iparMM <- function(A, B, comm=1, sleep=0.001){
+mpi.iparMM <- function(A, B, comm=1, sleep=0.01){
     splitRows <- function(x, ncl)
         lapply(.splitIndices(nrow(x), ncl), function(i) x[i,, drop=FALSE])    
     .docall(rbind, mpi.iapply(splitRows(A, mpi.comm.size(comm)-1), 
@@ -582,13 +591,14 @@ mpi.iparMM <- function(A, B, comm=1, sleep=0.001){
 }    
 
 mpi.applyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1){
+	apply.seq=NULL
     n <- length(x)
     slave.num <- mpi.comm.size(comm)-1
     if (slave.num < 1)
         stop("There are no slaves running")
     if (n <= slave.num) {
         if (exists(".mpi.applyLB")) 
-            rm(.mpi.applyLB,  envir=.GlobalEnv)
+			rm(".mpi.applyLB",  envir=.GlobalEnv)
         return (mpi.apply(x,fun,...,comm=comm))
     }    
     if (!is.function(fun))
@@ -601,8 +611,8 @@ mpi.applyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1){
                 length(apply.seq)!=n)
             stop("apply.seq is not in right order")
             
-    mpi.bcast.cmd(.mpi.slave.applyLB(),comm=comm)
-    mpi.bcast(as.integer(n),type=1,comm=comm)
+    mpi.bcast.cmd(.mpi.worker.applyLB, n=n, comm=comm)
+    #mpi.bcast(as.integer(n),type=1,comm=comm)
     mpi.bcast.Robj(list(fun=fun,dot.arg=list(...)),rank=0,comm=comm)
     out <- as.list(integer(n))
     mpi.anysource <- mpi.any.source()
@@ -623,24 +633,27 @@ mpi.applyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1){
         }
         return(out)
     }
-    .mpi.applyLB <<- integer(n)
+    #.mpi.applyLB <<- integer(n)
+	mpi.seq.tmp <- integer(n)
     for (i in 1:n){
        tmp<- mpi.recv.Robj(mpi.anysource,mpi.anytag,comm)
        srctag <- mpi.get.sourcetag()
        out[[srctag[2]]]<- tmp
-       .mpi.applyLB[i] <<- srctag[1]
+       mpi.seq.tmp[i] <- srctag[1]
        j <- i+slave.num
        if (j <= n)
             mpi.send.Robj(list(data.arg=list(x[[j]])), dest=srctag[1],tag=j, comm=comm)
        else
             mpi.send.Robj(as.integer(0),dest=srctag[1],tag=j,comm=comm)
     }
-    out
+ 	#assign(".mpi.applyLB",mpi.seq.tmp, envir = .GlobalEnv)
+	out
 }
 
-.mpi.slave.applyLB <- function(){
-    assign(".mpi.err", FALSE,  envir = .GlobalEnv)
-    n <- mpi.bcast(integer(1), type=1, comm=.comm)
+.mpi.worker.applyLB <- function(n){
+    #assign(".mpi.err", FALSE,  envir = .GlobalEnv)
+	.comm <- 1
+    #n <- mpi.bcast(integer(1), type=1, comm=.comm)
     tmpfunarg <- mpi.bcast.Robj(rank=0, comm=.comm)
     .tmpfun <- tmpfunarg$fun
     dotarg <- tmpfunarg$dot.arg
@@ -657,19 +670,20 @@ mpi.applyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1){
     }
 }
 
-mpi.iapplyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1, sleep=0.001){
+mpi.iapplyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1, sleep=0.01){
+	apply.seq=NULL
     n <- length(x)
     slave.num <- mpi.comm.size(comm)-1
     if (slave.num < 1)
         stop("There are no slaves running")
     if (n <= slave.num) {
         if (exists(".mpi.applyLB"))
-            rm(.mpi.applyLB,  envir=.GlobalEnv)
+            rm(".mpi.applyLB",  envir =.GlobalEnv)
         return (mpi.iapply(x,fun,...,comm=comm,sleep=sleep))
     }
     if (!is.function(fun))
         stop("fun is not a function")
-    if (slave.num > 10000)
+    if (slave.num > 2000)
         stop("Total slaves are more than nonblock send/receive can handle")
     length(list(...))
     if (!is.null(apply.seq))
@@ -679,8 +693,8 @@ mpi.iapplyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1, sleep=0.001){
                 length(apply.seq)!=n)
             stop("apply.seq is not in right order")
 
-    mpi.bcast.cmd(.mpi.slave.applyLB(),comm=comm)
-    mpi.bcast(as.integer(n),type=1,comm=comm)
+    mpi.bcast.cmd(.mpi.worker.applyLB, n=n, comm=comm)
+    #mpi.bcast(as.integer(n),type=1,comm=comm)
     mpi.bcast.Robj(list(fun=fun,dot.arg=list(...)),rank=0,comm=comm)
     out <- as.list(integer(n))
     mpi.anysource <- mpi.any.source()
@@ -702,7 +716,7 @@ mpi.iapplyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1, sleep=0.001){
                 mpi.send.Robj(as.integer(0),dest=apply.seq[i],tag=j,comm=comm)  
             charlen <- mpi.get.count(type=4)
             tag <- mpi.get.sourcetag()[2]
-            tmp <- .mpi.unserialize(mpi.recv(x = raw(charlen), type = 4, apply.seq[i], tag, comm))
+            tmp <- unserialize(mpi.recv(x = raw(charlen), type = 4, apply.seq[i], tag, comm))
             out[[tag]]<- tmp
             #mpi.wait(0)
         }
@@ -712,7 +726,7 @@ mpi.iapplyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1, sleep=0.001){
       }
       return(out)
     }
-    .mpi.applyLB <<- integer(n)
+    mpi.seq.tmp <- integer(n)
     i=0
     repeat {
         if (mpi.iprobe(mpi.anysource,mpi.anytag,comm)){
@@ -726,9 +740,9 @@ mpi.iapplyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1, sleep=0.001){
             else
                 mpi.send.Robj(as.integer(0),dest=src,tag=j,comm=comm)
             charlen <- mpi.get.count(type=4)
-            tmp <- .mpi.unserialize(mpi.recv(x = raw(charlen), type = 4, src, tag, comm))
+            tmp <- unserialize(mpi.recv(x = raw(charlen), type = 4, src, tag, comm))
             out[[tag]]<- tmp
-            .mpi.applyLB[i] <<- src
+            mpi.seq.tmp[i] <- src
             #mpi.wait(src-1)
         }
         if (i < n)
@@ -736,10 +750,11 @@ mpi.iapplyLB <- function(x, fun, ...,  apply.seq=NULL, comm=1, sleep=0.001){
         else
             break
     }
+  	#assign(".mpi.applyLB",mpi.seq.tmp, envir = .GlobalEnv)
     out
 }
 
-#.mpi.slave.iapplyLB <- function(){
+#.mpi.worker.iapplyLB <- function(){
 #    assign(".mpi.err", FALSE,  envir = .GlobalEnv)
 #    n <- mpi.bcast(integer(1), type=1, comm=.comm)
 #    tmpfunarg <- mpi.bcast.Robj(rank=0, comm=.comm)
@@ -783,7 +798,7 @@ mpi.parLapply <- function(x, fun, ..., job.num=mpi.comm.size(comm)-1, apply.seq=
         lapply, fun, ..., apply.seq=apply.seq, comm=comm))
 }
 
-mpi.iparLapply <- function(x, fun, ..., job.num=mpi.comm.size(comm)-1, apply.seq=NULL, comm=1, sleep=0.001){
+mpi.iparLapply <- function(x, fun, ..., job.num=mpi.comm.size(comm)-1, apply.seq=NULL, comm=1, sleep=0.01){
     if (job.num < 2)
         stop("job.num is at least 2.")
     splitList <- function(x, ncl)
@@ -803,7 +818,7 @@ mpi.parSapply <- function (x, fun, ..., job.num=mpi.comm.size(comm)-1, apply.seq
 }
 
 mpi.iparSapply <- function (x, fun, ..., job.num=mpi.comm.size(comm)-1, apply.seq=NULL, 
-                simplify = TRUE, USE.NAMES = TRUE, comm=1,sleep=0.001) 
+                simplify = TRUE, USE.NAMES = TRUE, comm=1,sleep=0.01) 
 {
     FUN <- match.fun(fun)
     answer <- mpi.iparLapply(as.list(x),FUN,...,job.num=job.num,apply.seq=apply.seq,comm=comm,sleep=sleep)
@@ -819,7 +834,7 @@ mpi.parReplicate <- function(n,  expr, job.num=mpi.comm.size(comm)-1, apply.seq=
 }
 
 mpi.iparReplicate <- function(n,  expr, job.num=mpi.comm.size(comm)-1, apply.seq=NULL,
-                                simplify = TRUE, comm=1,sleep=0.001){
+                                simplify = TRUE, comm=1,sleep=0.01){
     mpi.iparSapply(integer(n), eval.parent(substitute(function(...) expr)), 
     job.num=job.num, apply.seq=apply.seq, simplify = simplify, comm=comm,sleep=sleep)
 }
@@ -833,7 +848,7 @@ mpi.parRapply <- function(x,fun,...,job.num=mpi.comm.size(comm)-1,apply.seq=NULL
             apply.seq=apply.seq, comm=comm))
 }
 
-mpi.iparRapply <- function(x,fun,...,job.num=mpi.comm.size(comm)-1,apply.seq=NULL,comm=1,sleep=0.001){
+mpi.iparRapply <- function(x,fun,...,job.num=mpi.comm.size(comm)-1,apply.seq=NULL,comm=1,sleep=0.01){
     if (job.num < 2)
         stop("job.num is at least 2.")
     splitRows <- function(x, ncl)
@@ -851,7 +866,7 @@ mpi.parCapply <- function(x,fun,...,job.num=mpi.comm.size(comm)-1,apply.seq=NULL
             apply.seq=apply.seq, comm=comm))
 }
 
-mpi.iparCapply <- function(x,fun,...,job.num=mpi.comm.size(comm)-1,apply.seq=NULL,comm=1,sleep=0.001){
+mpi.iparCapply <- function(x,fun,...,job.num=mpi.comm.size(comm)-1,apply.seq=NULL,comm=1,sleep=0.01){
     if (job.num < 2)
         stop("job.num is at least 2.")
     splitCols <- function(x, ncl)
@@ -927,7 +942,7 @@ mpi.parApply <- function(x, MARGIN, fun, ..., job.num = mpi.comm.size(comm)-1,
 }
 
 mpi.iparApply <- function(x, MARGIN, fun, ..., job.num = mpi.comm.size(comm)-1,
-                    apply.seq=NULL, comm=1,sleep=0.001)
+                    apply.seq=NULL, comm=1,sleep=0.01)
 {
     FUN <- match.fun(fun)
     d <- dim(x)
@@ -992,4 +1007,4 @@ mpi.iparApply <- function(x, MARGIN, fun, ..., job.num = mpi.comm.size(comm)-1,
     return(ans)
 }
 
-mpi.parallel.sim <- mpi.parSim
+#mpi.parallel.sim <- mpi.parSim
