@@ -16,8 +16,12 @@
 ## phi: the IV estimator of phi(z) corresponding to the estimated
 ## derivative phihat(z)
 ## phi.prime: the IV derivative estimator
+## phi.mat: the matrix with colums phi_1, phi_2 etc. over all iterations
+## phi.prime.mat: the matrix with colums phi'_1, phi'_2 etc. over all iterations
 ## num.iterations: number of iterations taken by Landweber-Fridman
 ## norm.stop: the stopping rule for each Landweber-Fridman iteration
+## norm.value: the norm not multiplied by the number of iterations
+## convergence: a character string indicating whether/why iteration terminated
 
 ## First, a series of functions for local polynomial kernel regression
 ## Functions for generalized local polynomial regression
@@ -33,9 +37,9 @@
 ## where the local polynomial estimator is ill-conditioned (sparse
 ## data, small h etc.).
 
-## This function will compute the integral using the trapezoidal rule
-## and the cumsum function as we need to compute this in a
-## computationally efficient manner.
+## This function will compute the cumulative integral at each sample
+## realization using the trapezoidal rule and the cumsum function as
+## we need to compute this in a computationally efficient manner.
 
 integrate.trapezoidal <- function(x,y) {
   n <- length(x)
@@ -65,11 +69,11 @@ npregivderiv <- function(y,
                          optim.abstol=.Machine$double.eps,
                          optim.maxit=500,
                          iterate.max=1000,
-                         iterate.tol=1.0e-04,
                          iterate.diff.tol=1.0e-08,
                          constant=0.5,
-                         start.phi.zero=FALSE,
-                         smooth.while.iterating=TRUE,
+                         penalize.iteration=TRUE,
+                         start.from=c("Eyz","EEywz"),
+                         starting.values=NULL,
                          stop.on.increase=TRUE,
                          smooth.residuals=TRUE,
                          ...) {
@@ -345,12 +349,6 @@ npregivderiv <- function(y,
 
   }
 
-  ## No Zero Denominator, used in C code for kernel estimation...
-
-  NZD <- function(a) {
-    sapply(1:NROW(a), function(i) {if(a[i] < 0) min(-.Machine$double.xmin,a[i]) else max(.Machine$double.xmin,a[i])})
-  }
-
   mypoly <- function(X,degree) {
 
     if(missing(X)) stop(" X required")
@@ -1033,11 +1031,11 @@ npregivderiv <- function(y,
 
   ## Basic error checking
 
-  if(!is.logical(start.phi.zero)) stop("start.phi.zero must be logical (TRUE/FALSE)")
-  if(!is.logical(smooth.while.iterating)) stop("smooth.while.iterating must be logical (TRUE/FALSE)")
-  if(!is.logical(stop.on.increase)) stop("stop.on.increase must be logical (TRUE/FALSE)")  
-  if(!is.logical(smooth.residuals)) stop("smooth.residuals must be logical (TRUE/FALSE)")  
+  if(!is.logical(penalize.iteration)) stop("penalize.iteration must be logical (TRUE/FALSE)")
+  if(!is.logical(stop.on.increase)) stop("stop.on.increase must be logical (TRUE/FALSE)")
+  if(!is.logical(smooth.residuals)) stop("smooth.residuals must be logical (TRUE/FALSE)")
 
+  start.from <- match.arg(start.from)
   optim.method <- match.arg(optim.method)
 
   if(p < 0) stop("The order of the local polynomial must be a positive integer")
@@ -1047,7 +1045,6 @@ npregivderiv <- function(y,
   if(optim.abstol <= 0) stop("optim.abstol must be positive")
   if(optim.maxit <= 0) stop("optim.maxit must be a positive integer")
   if(iterate.max < 2) stop("iterate.max must be at least 2")
-  if(iterate.tol <= 0) stop("iterate.tol must be positive")
   if(iterate.diff.tol < 0) stop("iterate.diff.tol must be non-negative")
   if(constant <= 0 || constant >= 1) stop("constant must lie in the range (0,1)")
 
@@ -1064,6 +1061,8 @@ npregivderiv <- function(y,
   if(is.null(weval)) weval <- w
   if(is.null(weval)) xeval <- x
 
+  if(!is.null(starting.values) && (NROW(starting.values) != NROW(zeval))) stop(paste("starting.values must be of length",NROW(zeval)))
+
   ## Need to determine how many x, w, z are numeric
 
   z <- data.frame(z)
@@ -1107,10 +1106,35 @@ npregivderiv <- function(y,
   model.Sz <- npudist(tdat=z,bws=bw)
   S.z <- 1-predict(model.Sz,newdata=zeval)
 
+  console <- printClear(console)
+  console <- printPop(console)
+  console <- printPush(paste("Computing optimal smoothing for E(y|w) (stopping rule) for iteration 1...",sep=""),console)
+
+  ## For stopping rule...
+
+  hyw <- glpcv(ydat=y,
+               xdat=w,
+               degree=rep(p, num.w.numeric),
+               nmulti=nmulti,
+               random.seed=random.seed,
+               optim.maxattempts=optim.maxattempts,
+               optim.method=optim.method,
+               optim.reltol=optim.reltol,
+               optim.abstol=optim.abstol,
+               optim.maxit=optim.maxit,
+               ...)
+
+  E.y.w <- glpreg(tydat=y,
+                  txdat=w,
+                  exdat=weval,
+                  bws=hyw$bw,
+                  degree=rep(p, num.w.numeric),
+                  ...)$mean
+
   ## Potential alternative starting rule (consistent with
   ## npregiv). Here we start with E(Y|Z) rather than zero
 
-  if(!start.phi.zero) {
+  if(is.null(starting.values)) {
 
     console <- printClear(console)
     console <- printPop(console)
@@ -1120,7 +1144,7 @@ npregivderiv <- function(y,
       console <- printPush(paste("Computing optimal smoothing for E(y|z,x) for iteration 1...",sep=""),console)
     }
 
-    h <- glpcv(ydat=y,
+    h <- glpcv(ydat=if(start.from=="Eyz") y else E.y.w,
                xdat=z,
                degree=rep(p, num.z.numeric),
                nmulti=nmulti,
@@ -1136,7 +1160,7 @@ npregivderiv <- function(y,
 
       ## glpreg() does not provide local constant derivative
 
-      phi.prime <- gradients(npreg(tydat=y,
+      phi.prime <- gradients(npreg(tydat=if(start.from=="Eyz") y else E.y.w,
                                    txdat=z,
                                    exdat=zeval,
                                    bws=h$bw,
@@ -1144,7 +1168,7 @@ npregivderiv <- function(y,
 
     } else {
 
-      grad.object <- glpreg(tydat=y,
+      grad.object <- glpreg(tydat=if(start.from=="Eyz") y else E.y.w,
                             txdat=z,
                             exdat=zeval,
                             bws=h$bw,
@@ -1162,104 +1186,12 @@ npregivderiv <- function(y,
 
     }
 
-    ## Step 1 - begin iteration - for this we require \varphi_0. To
-    ## compute \varphi_{0,i}, we require \mu_{0,i}. For j=0 (first
-    ## term in the series), \mu_{0,i} is Y_i.
-
-    console <- printClear(console)
-    console <- printPop(console)
-    console <- printPush(paste("Computing optimal smoothing for E(y|w) (stopping rule) for iteration 1...",sep=""),console)
-
-    ## For stopping rule...
-    
-    hyw <- glpcv(ydat=y,
-                 xdat=w,
-                 degree=rep(p, num.w.numeric),
-                 nmulti=nmulti,
-                 random.seed=random.seed,
-                 optim.maxattempts=optim.maxattempts,
-                 optim.method=optim.method,
-                 optim.reltol=optim.reltol,
-                 optim.abstol=optim.abstol,
-                 optim.maxit=optim.maxit,
-                 ...)
-    
-    E.y.w <- glpreg(tydat=y,
-                    txdat=w,
-                    exdat=weval,
-                    bws=hyw$bw,
-                    degree=rep(p, num.w.numeric),
-                    ...)$mean
-
   } else {
-    
-    ## Step 1 - begin iteration - for this we require \varphi_0. To
-    ## compute \varphi_{0,i}, we require \mu_{0,i}. For j=0 (first
-    ## term in the series), \mu_{0,i} is Y_i.
-    
-    mu <- y
-    
-    ## We also require the mean of \miu_{0,i} shortly...
-    
-    mean.mu <- mean(mu)
-    
-    console <- printClear(console)
-    console <- printPop(console)
-    console <- printPush(paste("Computing optimal smoothing for E(y|w) (stopping rule) for iteration 1...",sep=""),console)
 
-    ## Next, we regress require \mu_{0,i} W (for first iteration mu is y)
-    
-    hyw <- glpcv(ydat=y,
-                 xdat=w,
-                 degree=rep(p, num.w.numeric),
-                 nmulti=nmulti,
-                 random.seed=random.seed,
-                 optim.maxattempts=optim.maxattempts,
-                 optim.method=optim.method,
-                 optim.reltol=optim.reltol,
-                 optim.abstol=optim.abstol,
-                 optim.maxit=optim.maxit,
-                 ...)
-    
-    E.y.w <- glpreg(tydat=y,
-                    txdat=w,
-                    exdat=weval,
-                    bws=hyw$bw,
-                    degree=rep(p, num.w.numeric),
-                    ...)$mean
-    
-    ## We require the mean of the fitted values
-    
-    mean.predicted.E.mu.w <- mean(E.y.w)
-    
-    ## We need the mean of the fitted values for this (we readily
-    ## compute the CDF not the survivor, so anything that is weighted
-    ## by the survivor kernel can be expressed as the mean of that
-    ## being weighted minus the weighting using the CDF kernel).
-    ## Next, we need the weighted sum of the survivor kernel where the
-    ## weights are E[\mu_{0,i}|W]. We can write this as the mean of the
-    ## \mu_{0,i} minus the weighted sum using the CDF kernel, i.e. if
-    ## K is a CDF kernel, then n^{-1}\sum_j \bar K() \mu_{0,i} =
-    ## n^{-1}\sum_j (1- K()) \mu_{0,i} = n^{-1}\sum_j\mu_{0,i}-
-    ## n^{-1}\sum_j K() \mu_{0,i}
-    
-    ## Now we compute T^* applied to E.y.w, and this is phi.prime.0 for
-    ## j=0.
-    
-    ## CDF weighted sum (but we need survivor weighted sum...)
-    
-    cdf.weighted.average <- npksum(txdat=z,
-                                   exdat=zeval,
-                                   tydat=as.matrix(E.y.w),
-                                   operator="integral",
-                                   bws=bw$bw)$ksum/length(y)
-    
-    survivor.weighted.average <- mean.predicted.E.mu.w - cdf.weighted.average
+    phi.prime <- starting.values
 
-    phi.prime <- (survivor.weighted.average - S.z*mean.mu)/f.z
+  }
 
-  }  
-  
   ## Now we can compute phi.0 by integrating phi.prime.0 up to each
   ## sample realization (here we use the trapezoidal rule)
 
@@ -1268,6 +1200,9 @@ npregivderiv <- function(y,
 
   phi <- integrate.trapezoidal(z[,1],phi.prime)
 
+  starting.values.phi <- phi
+  starting.values.phi.prime <- phi.prime
+
   ## In the definition of phi we have the integral minus the mean of
   ## the integral with respect to z, so subtract the mean here
 
@@ -1277,42 +1212,16 @@ npregivderiv <- function(y,
   console <- printPop(console)
   console <- printPush(paste("Computing optimal smoothing for E(phi|w) (stopping rule) for iteration 1...",sep=""),console)
 
-  ## For the stopping rule, we require E.phi.w
-
-  h.E.phi.w <- glpcv(ydat=phi,
-                     xdat=w,
-                     degree=rep(p, num.w.numeric),
-                     nmulti=nmulti,
-                     random.seed=random.seed,
-                     optim.maxattempts=optim.maxattempts,
-                     optim.method=optim.method,
-                     optim.reltol=optim.reltol,
-                     optim.abstol=optim.abstol,
-                     optim.maxit=optim.maxit,
-                     ...)
-
-  E.phi.w <- glpreg(tydat=phi,
-                    txdat=w,
-                    eydat=phi,
-                    exdat=weval,
-                    bws=h.E.phi.w$bw,
-                    degree=rep(p, num.w.numeric),
-                    ...)$mean
-
   norm.stop <- numeric()
 
-  norm.stop[1] <- mean(((E.y.w-E.phi.w)/E.y.w)^2)
-
   ## Now we compute mu.0 (a residual of sorts)
 
   mu <- y - phi
 
   ## Now we repeat this entire process using mu = y = phi.0 rather than y
 
-  mean.mu <- mean(mu)
-
   if(smooth.residuals) {
-    
+
     console <- printClear(console)
     console <- printPop(console)
     console <- printPush(paste("Computing optimal smoothing for E(mu|w) (stopping rule) for iteration 1...",sep=""),console)
@@ -1321,7 +1230,7 @@ npregivderiv <- function(y,
     ## we have computed the stopping rule so reuse the bandwidth
     ## vector to be passed below. Here we compute the bandwidth
     ## optimal for the regression of mu on w.
-    
+
     h.E.mu.w <- glpcv(ydat=mu,
                        xdat=w,
                        degree=rep(p, num.w.numeric),
@@ -1333,7 +1242,7 @@ npregivderiv <- function(y,
                        optim.abstol=optim.abstol,
                        optim.maxit=optim.maxit,
                        ...)
-    
+
     ## Next, we regress require \mu_{0,i} W using bws optimal for phi on w
 
     predicted.E.mu.w <- glpreg(tydat=mu,
@@ -1346,10 +1255,32 @@ npregivderiv <- function(y,
 
   } else {
 
+      h.E.phi.w <- glpcv(ydat=phi,
+                         xdat=w,
+                         degree=rep(p, num.w.numeric),
+                         nmulti=nmulti,
+                         random.seed=random.seed,
+                         optim.maxattempts=optim.maxattempts,
+                         optim.method=optim.method,
+                         optim.reltol=optim.reltol,
+                         optim.abstol=optim.abstol,
+                         optim.maxit=optim.maxit,
+                         ...)
+
+      E.phi.w <- glpreg(tydat=phi,
+                        txdat=w,
+                        eydat=phi,
+                        exdat=weval,
+                        bws=h.E.phi.w$bw,
+                        degree=rep(p, num.w.numeric),
+                        ...)$mean
+
     predicted.E.mu.w <- E.y.w - E.phi.w
 
   }
 
+  norm.stop[1] <- sum(predicted.E.mu.w^2)/sum(E.y.w^2)
+
   ## We again require the mean of the fitted values
 
   mean.predicted.E.mu.w <- mean(predicted.E.mu.w)
@@ -1364,32 +1295,23 @@ npregivderiv <- function(y,
 
   survivor.weighted.average <- mean.predicted.E.mu.w - cdf.weighted.average
 
-  T.star.mu <- (survivor.weighted.average-S.z*mean.mu)/f.z
+  T.star.mu <- (survivor.weighted.average-S.z*mean.predicted.E.mu.w)/f.z
 
   ## Now we update phi.prime.0, this provides phi.prime.1, and now
   ## we can iterate until convergence... note we replace phi.prime.0
   ## with phi.prime.1 (i.e. overwrite phi.prime)
 
   phi.prime <- phi.prime + constant*T.star.mu
+  phi.mat <- phi
+  phi.prime.mat <- phi.prime
 
   ## This we iterate...
 
   for(j in 2:iterate.max) {
 
-    ## Save previous in case stop norm increases
-
-    phi.j.m.1 <- phi
-    phi.prime.j.m.1 <- phi.prime
-
-    if(smooth.while.iterating) {
-      console <- printClear(console)
-      console <- printPop(console)
-      console <- printPush(paste("Computing optimal smoothing for E(phi|w) for iteration ", j,"...",sep=""),console)
-    } else {
-      console <- printClear(console)
-      console <- printPop(console)
-      console <- printPush(paste("Computing E(phi|w) for iteration ", j,"...",sep=""),console)
-    }
+    console <- printClear(console)
+    console <- printPop(console)
+    console <- printPush(paste("Computing optimal smoothing for E(phi|w) for iteration ", j,"...",sep=""),console)
 
     ## NOTE - this presumes univariate z case... in general this would
     ## be a continuous variable's index
@@ -1401,34 +1323,6 @@ npregivderiv <- function(y,
 
     phi <- phi - mean(phi) + mean(y)
 
-    ## For the stopping rule, we require E.phi.w
-
-    if(smooth.while.iterating) {
-
-        h.E.phi.w <- glpcv(ydat=phi,
-                           xdat=w,
-                           degree=rep(p, num.w.numeric),
-                           nmulti=nmulti,
-                           random.seed=random.seed,
-                           optim.maxattempts=optim.maxattempts,
-                           optim.method=optim.method,
-                           optim.reltol=optim.reltol,
-                           optim.abstol=optim.abstol,
-                           optim.maxit=optim.maxit,
-                           ...)
-
-    }
-
-    E.phi.w <- glpreg(tydat=phi,
-                      txdat=w,
-                      eydat=phi,
-                      exdat=weval,
-                      bws=h.E.phi.w$bw,
-                      degree=rep(p, num.w.numeric),
-                      ...)$mean
-
-    norm.stop[j] <- mean(((E.y.w-E.phi.w)/E.y.w)^2)
-
     ## Now we compute mu.0 (a residual of sorts)
 
     mu <- y - phi
@@ -1438,16 +1332,16 @@ npregivderiv <- function(y,
     mean.mu <- mean(mu)
 
     if(smooth.residuals) {
-    
+
       console <- printClear(console)
       console <- printPop(console)
       console <- printPush(paste("Computing optimal smoothing for E(mu|w) for iteration ", j,"...",sep=""),console)
-      
+
       ## Additional smoothing on top of the stopping rule required, but
       ## we have computed the stopping rule so reuse the bandwidth
       ## vector to be passed below. Here we compute the bandwidth
       ## optimal for the regression of mu on w.
-      
+
       h.E.mu.w <- glpcv(ydat=mu,
                         xdat=w,
                         degree=rep(p, num.w.numeric),
@@ -1459,9 +1353,9 @@ npregivderiv <- function(y,
                         optim.abstol=optim.abstol,
                         optim.maxit=optim.maxit,
                         ...)
-      
+
       ## Next, we regress require \mu_{0,i} W using bws optimal for phi on w
-      
+
       predicted.E.mu.w <- glpreg(tydat=mu,
                                  txdat=w,
                                  eydat=mu,
@@ -1469,13 +1363,35 @@ npregivderiv <- function(y,
                                  bws=h.E.mu.w$bw,
                                  degree=rep(p, num.w.numeric),
                                  ...)$mean
-      
+
     } else {
-      
+
+      h.E.phi.w <- glpcv(ydat=phi,
+                         xdat=w,
+                         degree=rep(p, num.w.numeric),
+                         nmulti=nmulti,
+                         random.seed=random.seed,
+                         optim.maxattempts=optim.maxattempts,
+                         optim.method=optim.method,
+                         optim.reltol=optim.reltol,
+                         optim.abstol=optim.abstol,
+                         optim.maxit=optim.maxit,
+                         ...)
+
+      E.phi.w <- glpreg(tydat=phi,
+                        txdat=w,
+                        eydat=phi,
+                        exdat=weval,
+                        bws=h.E.phi.w$bw,
+                        degree=rep(p, num.w.numeric),
+                        ...)$mean
+
       predicted.E.mu.w <- E.y.w - E.phi.w
-      
+
     }
-    
+
+    norm.stop[j] <- ifelse(penalize.iteration,j*sum(predicted.E.mu.w^2)/sum(E.y.w^2),sum(predicted.E.mu.w^2)/sum(E.y.w^2))
+
     mean.predicted.E.mu.w <- mean(predicted.E.mu.w)
 
     ## Now we compute T^* applied to mu
@@ -1494,18 +1410,68 @@ npregivderiv <- function(y,
     ## iterate until convergence...
 
     phi.prime <- phi.prime + constant*T.star.mu
+    phi.mat <- cbind(phi.mat,phi)
+    phi.prime.mat <- cbind(phi.prime.mat,phi.prime)
 
-    ## If stopping rule criterion increases or we are below stopping
-    ## tolerance then break
+    ## The number of iterations in LF is asymptotically equivalent to
+    ## 1/alpha (where alpha is the regularization parameter in
+    ## Tikhonov).  Plus the criterion function we use is increasing
+    ## for very small number of iterations. So we need a threshold
+    ## after which we can pretty much confidently say that the
+    ## stopping criterion is decreasing.  In Darolles et al. (2011)
+    ## \alpha ~ O(N^(-1/(min(beta,2)+2)), where beta is the so called
+    ## qualification of your regularization method. Take the worst
+    ## case in which beta = 0 and then the number of iterations is ~
+    ## N^0.5. Note that derivative estimation seems to require more
+    ## iterations hence the heuristic sqrt(N)
+
+    if(j > round(sqrt(nrow(z))) && !is.monotone.increasing(norm.stop)) {
+
+      ## If stopping rule criterion increases or we are below stopping
+      ## tolerance then break
+
+      if(stop.on.increase && norm.stop[j] > norm.stop[j-1]) {
+        convergence <- "STOP_ON_INCREASE"
+        break()
+      }
+      if(abs(norm.stop[j-1]-norm.stop[j]) < iterate.diff.tol) {
+        convergence <- "ITERATE_DIFF_TOL"
+        break()
+      }
 
-    if(norm.stop[j] < iterate.tol) break()
-    if(norm.stop[j-1] - norm.stop[j] < iterate.diff.tol) break()
-    if(stop.on.increase && norm.stop[j] > norm.stop[j-1]) {
-      phi <- phi.j.m.1 
-      phi.prime <- phi.prime.j.m.1
-      break()
     }
 
+    convergence <- "ITERATE_MAX"
+
+  }
+
+  ## Extract minimum, and check for monotone increasing function and
+  ## issue warning in that case. Otherwise allow for an increasing
+  ## then decreasing (and potentially increasing thereafter) portion
+  ## of the stopping function, ignore the initial increasing portion,
+  ## and take the min from where the initial inflection point occurs
+  ## to the length of norm.stop
+
+  norm.value <- norm.stop/(1:length(norm.stop))
+
+  if(which.min(norm.stop) == 1 && is.monotone.increasing(norm.stop)) {
+    warning("Stopping rule increases monotonically (consult model$norm.stop):\nThis could be the result of an inspired initial value (unlikely)\nNote: we suggest manually choosing phi.0 and restarting (e.g. instead set `starting.values' to E[E(Y|w)|z])")
+    convergence <- "FAILURE_MONOTONE_INCREASING"
+#    phi <- starting.values.phi
+#    phi.prime <- starting.values.phi.prime
+    j <- 1
+    while(norm.value[j+1] > norm.value[j]) j <- j + 1
+    j <- j-1 + which.min(norm.value[j:length(norm.value)])
+    phi <- phi.mat[,j]
+    phi.prime <- phi.prime.mat[,j]
+  } else {
+    ## Ignore the initial increasing portion, take the min to the
+    ## right of where the initial inflection point occurs
+    j <- 1
+    while(norm.stop[j+1] > norm.stop[j]) j <- j + 1
+    j <- j-1 + which.min(norm.stop[j:length(norm.stop)])
+    phi <- phi.mat[,j]
+    phi.prime <- phi.prime.mat[,j]
   }
 
   console <- printClear(console)
@@ -1513,7 +1479,16 @@ npregivderiv <- function(y,
 
   if(j == iterate.max) warning(" iterate.max reached: increase iterate.max or inspect norm.stop vector")
 
-  return(list(phi=phi,phi.prime=phi.prime,num.iterations=j,norm.stop=norm.stop))
+  return(list(phi=phi,
+              phi.prime=phi.prime,
+              phi.mat=phi.mat,
+              phi.prime.mat=phi.prime.mat,
+              num.iterations=j,
+              norm.stop=norm.stop,
+              norm.value=norm.value,
+              convergence=convergence,
+              starting.values.phi=starting.values.phi,
+              starting.values.phi.prime=starting.values.phi.prime))
 
 }
 
