@@ -119,15 +119,10 @@ double **matrix_bandwidth_deriv)
 			if(vec_sdev_x[i] <= DBL_MIN)
 			{
 #ifdef MPI2
-				if(my_rank == 0) {
-#endif
-				REprintf("\r ** Fatal Error in routine kernel_bandwidth() ** The variable appears to be constant!");
-#ifdef MPI2
-				}
 				/* Since we are exiting and this will terminate the program, clean up */
 				MPI_Finalize();
 #endif
-				error("\n ** Program terminated abnormally!\n");
+				error("\r ** Fatal Error in routine kernel_bandwidth() ** The variable appears to be constant!");
 			}
 
 		}
@@ -140,15 +135,10 @@ double **matrix_bandwidth_deriv)
 			if(vec_sdev_y[i] <= DBL_MIN)
 			{
 #ifdef MPI2
-				if(my_rank == 0) {
-#endif
-				REprintf("\r ** Fatal Error in routine kernel_bandwidth() ** The variable appears to be constant!");
-#ifdef MPI2
-				}
 				/* Since we are exiting and this will terminate the program, clean up */
 				MPI_Finalize();
 #endif
-				error("\n ** Program terminated abnormally!\n");
+				error("\r ** Fatal Error in routine kernel_bandwidth() ** The variable appears to be constant!");
 			}
 
 		}
@@ -581,15 +571,10 @@ fact constant. */
 			if(vec_sdev_x[i] <= DBL_MIN)
 			{
 #ifdef MPI2
-				if(my_rank == 0) {
-#endif
-				REprintf("\r ** Fatal Error in routine kernel_bandwidth() ** The variable appears to be constant!");
-#ifdef MPI2
-				}
 				/* Since we are exiting and this will terminate the program, clean up */
 				MPI_Finalize();
 #endif
-				error("\n ** Program terminated abnormally!\n");
+				error("\r ** Fatal Error in routine kernel_bandwidth() ** The variable appears to be constant!");
 			}
 
 		}
@@ -602,15 +587,10 @@ fact constant. */
 			if(vec_sdev_y[i] <= DBL_MIN)
 			{
 #ifdef MPI2
-				if(my_rank == 0) {
-#endif
-				REprintf("\r ** Fatal Error in routine kernel_bandwidth() ** The variable appears to be constant!");
-#ifdef MPI2
-				}
 				/* Since we are exiting and this will terminate the program, clean up */
 				MPI_Finalize();
 #endif
-				error("\n ** Program terminated abnormally!\n");
+				error("\r ** Fatal Error in routine kernel_bandwidth() ** The variable appears to be constant!");
 			}
 
 		}
