@@ -98,6 +98,8 @@ extern double *vector_scale_factor_extern;
 extern double y_min_extern;
 extern double y_max_extern;
 
+// cdf extern
+extern int int_fast_dls_extern;
 #ifdef RCSID
 static char rcsid[] = "$Id: kernelcv.c,v 1.9 2006/11/02 16:56:49 tristen Exp $";
 #endif
@@ -133,10 +135,6 @@ double cv_func_regression_categorical_ls(double *vector_scale_factor){
         return(DBL_MAX);
     }
 
-/* Compute the cross-validation function */
-/* Tristen's efficient code is broken under MPI, so we comment it out
-   here pending a fix */
-#ifndef MPI2
     if((BANDWIDTH_reg_extern == BW_FIXED)||(int_ll_extern == LL_LC)){ 
       return(np_kernel_estimate_regression_categorical_ls_aic(
         int_ll_extern,
@@ -156,11 +154,8 @@ double cv_func_regression_categorical_ls(double *vector_scale_factor){
         &vector_scale_factor[1],
         num_categories_extern));
       } else {
-#endif
       return(cv_func_regression_categorical_ls_nn(vector_scale_factor));
-#ifndef MPI2
       }
-#endif
 }
 
 double cv_func_regression_categorical_ls_nn(double *vector_scale_factor)
@@ -294,6 +289,71 @@ double cv_func_density_categorical_ml(double *vector_scale_factor)
 
 }
 
+double cv_func_con_distribution_categorical_ls(double *vector_scale_factor)
+{
+
+/* Numerical recipes wrapper function for likelihood density
+                    cross-validation */
+
+/* Declarations */
+
+    double cv = 0.0;
+
+    if(check_valid_scale_factor_cv(
+        KERNEL_den_extern,
+        KERNEL_reg_unordered_extern, /* Only for conditioning vars in conditional den */
+        BANDWIDTH_den_extern,
+        BANDWIDTH_den_extern,
+        0,
+        num_obs_train_extern,
+        num_var_continuous_extern,
+        num_var_unordered_extern,
+        num_var_ordered_extern,
+        num_reg_continuous_extern,
+        num_reg_unordered_extern,
+        num_reg_ordered_extern,
+        num_categories_extern,
+        vector_scale_factor) == 1) return(DBL_MAX);
+
+/* Compute the cross-validation function */
+
+    if(np_kernel_estimate_con_distribution_categorical_leave_one_out_ls_cv(KERNEL_den_extern,
+                                                                        KERNEL_den_unordered_extern,
+                                                                        KERNEL_den_ordered_extern,
+                                                                        KERNEL_reg_extern,
+                                                                        KERNEL_reg_unordered_extern,
+                                                                        KERNEL_reg_ordered_extern,
+                                                                        BANDWIDTH_den_extern,
+                                                                        num_obs_train_extern,
+                                                                        num_obs_eval_extern,
+                                                                        num_var_unordered_extern,
+                                                                        num_var_ordered_extern,
+                                                                        num_var_continuous_extern,
+                                                                        num_reg_unordered_extern,
+                                                                        num_reg_ordered_extern,
+                                                                        num_reg_continuous_extern,
+                                                                        int_fast_dls_extern,
+                                                                        matrix_Y_unordered_train_extern,
+                                                                        matrix_Y_ordered_train_extern,
+                                                                        matrix_Y_continuous_train_extern,
+                                                                        matrix_X_unordered_train_extern,
+                                                                        matrix_X_ordered_train_extern,
+                                                                        matrix_X_continuous_train_extern,
+                                                                        matrix_Y_unordered_eval_extern,
+                                                                        matrix_Y_ordered_eval_extern,
+                                                                        matrix_Y_continuous_eval_extern,
+                                                                        &vector_scale_factor[1],
+                                                                        num_categories_extern,
+                                                                        matrix_categorical_vals_extern,
+                                                                        &cv)==1)
+      {
+        return(DBL_MAX);
+      }
+
+
+    return(cv);
+
+}
 
 double cv_func_con_density_categorical_ml(double *vector_scale_factor)
 {
@@ -587,6 +647,63 @@ double cv_func_density_categorical_ls(double *vector_scale_factor)
 
 }
 
+double cv_func_distribution_categorical_ls(double *vector_scale_factor)
+{
+
+/* Numerical recipes wrapper function for likelihood density
+                    cross-validation */
+
+/* Declarations */
+
+    double cv = 0.0;
+
+    if(check_valid_scale_factor_cv(
+        KERNEL_den_extern,
+        KERNEL_den_unordered_extern,
+        BANDWIDTH_den_extern,
+        BANDWIDTH_den_extern,
+        0,
+        num_obs_train_extern,
+        0,
+        0,
+        0,
+        num_reg_continuous_extern,
+        num_reg_unordered_extern,
+        num_reg_ordered_extern,
+        num_categories_extern,
+        vector_scale_factor) == 1) return(DBL_MAX);
+
+/* Compute the cross-validation function */
+
+    if(np_kernel_estimate_distribution_ls_cv(KERNEL_den_extern,
+        KERNEL_den_unordered_extern,
+        KERNEL_den_ordered_extern,
+        BANDWIDTH_den_extern,
+        num_obs_train_extern,
+        num_obs_eval_extern,
+        num_reg_unordered_extern,
+        num_reg_ordered_extern,
+        num_reg_continuous_extern,
+        int_fast_dls_extern,
+        matrix_X_unordered_train_extern,
+        matrix_X_ordered_train_extern,
+        matrix_X_continuous_train_extern,
+        matrix_X_unordered_eval_extern,
+        matrix_X_ordered_eval_extern,
+        matrix_X_continuous_eval_extern,
+        &vector_scale_factor[1],
+        num_categories_extern,
+        matrix_categorical_vals_extern,
+        &cv)==1)
+    {
+        return(DBL_MAX);
+    }
+
+
+    return(cv);
+
+}
+
 
 double func_con_density_quantile(double *quantile)
 {
@@ -678,10 +795,6 @@ double cv_func_regression_categorical_aic_c(double *vector_scale_factor)
         return(DBL_MAX);
     }
 
-/* Compute the AIC_c function */
-/* Tristen's efficient code is broken under MPI, so we comment it out
-   here pending a fix */
-#ifndef MPI2
     if((BANDWIDTH_reg_extern == BW_FIXED)||(int_ll_extern == LL_LC)){
       return(np_kernel_estimate_regression_categorical_ls_aic(
         int_ll_extern,
@@ -701,7 +814,6 @@ double cv_func_regression_categorical_aic_c(double *vector_scale_factor)
         &vector_scale_factor[1],
         num_categories_extern));
         } else {
-#endif
       return(kernel_estimate_regression_categorical_aic_c(
         int_ll_extern,
         KERNEL_reg_extern,
@@ -718,8 +830,6 @@ double cv_func_regression_categorical_aic_c(double *vector_scale_factor)
         vector_Y_extern,
         &vector_scale_factor[1],
         num_categories_extern));
-#ifndef MPI2
     }
-#endif
 
 }
